#!/bin/bash

KL_LA_DIR="LaunchAgents/"
KL_LD_DIR="LaunchDaemons/"
KL_LA_USER_DIR="LaunchAgentsUser/"
KL_LOGIN_ITEMS="login-items"
KL_SAFARI_EXT_PLIST="safari-exts.plist"
KL_SAFARI_EXT="safari-exts"
KL_CHROME_EXT="chrome-exts"
KL_INSTALLED_PKGS='installed-packages'
KL_INSTALLED_PKGS_MD5='installed-files-md5checksums'
KL_DUMPBTM_FILE="dumpbtm"

LA_DIR="/Library/LaunchAgents/"
LD_DIR="/Library/LaunchDaemons/"
LA_USER_DIR="${HOME}/Library/LaunchAgents/"

SAFARI_EXT_DIR="${HOME}/Library/Safari/Extensions/"
SAFARI_EXT_PLIST="${SAFARI_EXT_DIR}Extensions.plist"

CHROME_EXT="${HOME}/Library/Application Support/Google/Chrome/Default/Extensions/"

#arg1: src_path; arg2: dest_path
function copy_files () {
	rm -rf "$2"
	mkdir "$2"
	
	if [ -d "$1" ]; then
		for file in ${1}/*; do
			cp "$file" "$2"
		done
	fi
}

echo "Collecting autolaunch plists..."

copy_files $LA_USER_DIR $KL_LA_USER_DIR
copy_files $LA_DIR $KL_LA_DIR
copy_files $LD_DIR $KL_LD_DIR

echo "Collecting login items props..."

osascript -e 'tell application "System Events" to get properties of every login item' > $KL_LOGIN_ITEMS

if [[ ${OSTYPE:6} -ge 22 ]]; then
	sfltool dumpbtm >> $KL_DUMPBTM_FILE
fi

echo "Collecting Safari extensions hashes..."

if [ -d "$SAFARI_EXT_DIR" ]; then
	if [ -f "$SAFARI_EXT_PLIST" ]; then
		cp "$SAFARI_EXT_PLIST" "$KL_SAFARI_EXT_PLIST"
		plutil -convert xml1 "$KL_SAFARI_EXT_PLIST"
	fi
	
	find "$SAFARI_EXT_DIR" -type f -print0 | xargs -0 md5 > $KL_SAFARI_EXT
fi

echo "Collecting Chrome extensions hashes..."

if [ -d "$CHROME_EXT" ]; then
	find "$CHROME_EXT" -type f -print0 | xargs -0 md5 > $KL_CHROME_EXT
fi

echo "Collecting installed packages hashes..."

rm -f $KL_INSTALLED_PKGS
rm -f $KL_INSTALLED_PKGS_MD5
for pkg in $(pkgutil --pkgs); do
	echo "$pkg" >> $KL_INSTALLED_PKGS
	pkgutil --only-files --files "$pkg" | sed 's/^/\//' | grep /MacOS/ | sed 's/ /\\ /g' | xargs md5 >> $KL_INSTALLED_PKGS_MD5
done

echo "Packing collected files..."

tar zcf ~/Desktop/lk.tar.gz $KL_LA_DIR $KL_LD_DIR $KL_LA_USER_DIR $KL_LOGIN_ITEMS $KL_SAFARI_EXT_PLIST $KL_SAFARI_EXT $KL_CHROME_EXT $KL_INSTALLED_PKGS $KL_INSTALLED_PKGS_MD5 $KL_DUMPBTM_FILE
rm -rf $KL_LA_DIR 
rm -rf $KL_LD_DIR 
rm -rf $KL_LA_USER_DIR 
rm -f $KL_LOGIN_ITEMS 
rm -f $KL_SAFARI_EXT_PLIST 
rm -f $KL_SAFARI_EXT 
rm -f $KL_CHROME_EXT 
rm -f $KL_INSTALLED_PKGS 
rm -f $KL_INSTALLED_PKGS_MD5
rm -f $KL_DUMPBTM_FILE

echo "Done!"